﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Model
{
    /// <summary>
    /// This class calculates all damage related calculations
    /// </summary>
    static class DamageCalculations
    {
        /**
        passes in the 2 units, and a boolean on whether attack is physical (false), or magical (true), and returns damage dealt by taking into account an attacker's str/int, and defender def/res
        *@param attacker The unit performing the attack
        *@param defender The unit defending against the attack
        *@param physOrMagic Boolean that tells the controller if it's physical or magical damage to be calculated
        */
        public static int getDamageDealt(Unit attacker, Unit defender, bool physOrMagic)
        {
            if (physOrMagic == false)
            {
                int damage = attacker.Str - defender.Def;
                if (damage < 0)// to prevent attacks from healing if defense is higher then attack
                {
                    damage = 0;
                }
                return damage; //return physical damage dealt
            }
            else
            {
                return attacker.Int - defender.Res; //else return magical damage dealt
            }
        }

        /**
        passes in the 2 units, and returns the hit rate as a percentage out of 100 by taking into account both unit's skill
        *@param attacker The unit performing the attack
        *@param defender The unit defending against the attack
        */
        public static int getHitRate(Unit attacker, Unit defender)
        {
            int hitRate = (int)Math.Round((((attacker.Skill / 10.0) - (defender.Skill / 10.0) + 1.0) * 0.8) * 100.0);
            if (hitRate > 100) //to prevent hit rate over 100%
            {
                hitRate = 100;
            }
            else if (hitRate < 0) //to prevent hit rate under 0%
            {
                hitRate = 0;
            }
            return hitRate;
        }

        /**
        passes in the 2 units, and returns the crit rate as a percentage out of 100 by taking into account both unit's skill
        *@param attacker The unit performing the attack
        *@param defender The unit defending against the attack
        */
        public static int getCritRate(Unit attacker, Unit defender)
        {

            int critRate = (int)Math.Round((((attacker.Skill / 3.0) - (defender.Skill / 3.0) + 1.0) * 0.1) * 100.0);
            if (critRate > 100) //to prevent crit rate over 100%
            {
                critRate = 100;
            }
            else if (critRate < 0) //to prevent crit rate under 0%
            {
                critRate = 0;
            }
            return critRate;
        }

        /**
        passes in then 2 units, and determines how many attacks the attacker makes by factoring in both unit's relative speed
        *@param attacker The unit performing the attack
        *@param defender The unit defending against the attack
        */
        public static int getHitCount(Unit attacker, Unit defender)
        {
            if (attacker.Speed > (defender.Speed + 4))
            {
                return 2; //return 2 attack if speed of attacker if 4 higher
            }
            else
            {
                return 1; //else return 1
            }
        }
        /**
        factors in damage dealt, hit rate, crit rate, and number of attacks (as in how above functions were calculated) to calculate actual damage dealt
        *@param attacker The unit performing the attack
        *@param defender The unit defending against the attack
        *@param physOrMagic Boolean that tells the controller if it's physical or magical damage to be calculated
        */
        public static int finalDamage(Unit attacker, Unit defender, bool physOrMagic)
        {
            int rawDamage = getDamageDealt(attacker, defender, physOrMagic);
            int hitRate = getHitRate(attacker, defender);
            int critRate = getCritRate(attacker, defender);
            int numOfAttacks = getHitCount(attacker, defender);

            Random rnd = new Random();
            int hitOrMiss = rnd.Next(0, 101); // creates a number between 0-100
            int critOrNot = rnd.Next(0, 101); // creates a number between 0-100

            if (hitOrMiss > hitRate) //if the random number is greater then the hitrate, the attack misses and 0 is returned
            {
                return 0;
            }
            else
            {
                if (critOrNot > critRate) //if attack doesn't crit
                {
                    return rawDamage * numOfAttacks; //return the damage * number of attacks calculated from above
                }
                else //else if attack crits
                {
                    return rawDamage * numOfAttacks * 2; //returns the damage * number of attacks *2 for critical for damage dealt elsewise
                }
            }
        }
    }
}
