﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
using View;

/// <summary>
/// The model in MVC. These classes contain the structure of the game, and will be controlled by Controller, and displayed in View.
/// </summary>

namespace Model
{
    /// <summary>
    /// Unit Interface for Warrior, Mage, and Archer 
    /// </summary>

    /**
    * This is the interface for the 3 playable unit classes, where the only
    differences in the units will be their statistics and graphical assets
    */
    interface Unit
    {
        #region Stats
        /**
        Sets and returns whether or not unit is alive
        */
        bool Alive { get; set; }

        /**
        Sets and returns a unit's HP
        */
        int Hp { get; set; }

        /**
        Sets and returns a unit's Strength
        */
        int Str { get; set; }

        /**
        Sets and returns a unit's Intelliegence
        */
        int Int { get; set; }

        /**
        Sets and returns a unit's Skill
        */
        int Skill { get; set; }

        /**
        Sets and returns a unit's Speed
        */
        int Speed { get; set; }

        /**
        Sets and returns a unit's Defense
        */
        int Def { get; set; }

        /**
        Sets and returns a unit's Resistance
        */
        int Res { get; set; }

        /**
        Sets and returns a unit's Level
        */
        int Level { get; set; }

        /**
        Returns the unit's movability range on grid (number of spaces the unit can move in one turn)
        */
        int getMovability();

        /**
        returns all stats as an array
        */
        int[] getStats();

        /**
        sets initial unit stats upon creation
        */
        void setInitialStats();
        #endregion

        /**
        returns weapon the unit is currently equipping
        */
        Weapon equippedWeapon { get; set; }
        //void setEquipableWeapons(Weapon add);  // need to update the weapon array, put new weapon into it
        /**
        indicates whether a button has already been previously selected or not
        */
        bool isButtonActive(ButtonType buttonType);

        /**
        sets the coordinates of menu buttons
        * @param pixelCoordinates The pixel coordinate of the button
        */
        void setButtonCoordinates(Vector2 pixelCoordinates);

        /**
        Sets and gets the current frame of the animation sequence
        */
        int currentFrame { get; set; }

        /**
        returns the sprite image of the unit
        */
        Texture2D getSpriteImage();

        /**
        returns the button texture at index i
        */
        Texture2D getButtonImage(ButtonType buttonType);

        /**
        returns the char info screen texture
        */
        Texture2D getCharInfo();

        /**
        returns the char attack info screen texture
        */
        Texture2D getCharAttackInfo();

        /**
        gets and sets unit's position by tile
        */
        Tuple<int, int> Position { get; set; }

        /**
        returns the pixel coordinate of the unit
        */
        Vector2 PixelCoordinates { get; set; }

        /**
        returns the dropdown menu buttons of the unit
        */
        Button[] getButtons();

        /**
        Returns the button type
        * @param buttonType The button to return (Move, Attack, Item, Wait, and attack confirm)
        */
        Button getButtonType(ButtonType buttonType);

        /**
        returns the current sprite frame in animation sequence
        */
        Rectangle getCurrentFrame();

        /**
        returns array of equipable weapons
        */
        Weapon[] getEquipableWeapons();

        /**
        returns unit's class (warrior, mage, archer)
        */
        UnitType getClass();

    }
    /**
    defines the possible classes of a unit
    */
    enum UnitType { Warrior, Archer, Mage };
    enum Direction { Down, Left, Right, Up };
}
